import 'package:clean_android_design/clean_android_design.dart';
import 'package:flutter/material.dart';

class NavigatorWidget extends StatefulWidget {
  final Widget menuWidget;

  const NavigatorWidget({
    Key key,
    this.menuWidget,
  }) : super(key: key);

  @override
  _NavigatorWidgetState createState() => _NavigatorWidgetState();
}

class _NavigatorWidgetState extends State<NavigatorWidget> {
  final RouteObserver<PageRoute> routeObserver = RouteObserver<PageRoute>();
  final RouteObserver<PageRoute> draggableRouteObserver =
      RouteObserver<PageRoute>();

  bool showRightSide = false;
  bool showDraggable = false;

  double draggableWidth, draggableHeight;
  Offset draggablePosition;

  @override
  Widget build(BuildContext context) {
    SplitNavigator.mainContext = context;

    return Stack(
      children: [
        _buildMainNavigators(),
        _buildDraggable(),
      ],
    );
  }

  Widget _buildMainNavigators() {
    return Scaffold(
      body: Row(
        children: <Widget>[
          widget.menuWidget ?? Container(),
          Expanded(
            child: LockedNavigatorWidget(
              child: Navigator(
                key: SplitNavigator.leftNavigatorKey,
                onGenerateRoute: (settings) => MaterialPageRoute(
                  builder: (context) {
                    return Container();
                  },
                ),
              ),
            ),
          ),
          Container(
            width: 20,
            height: MediaQuery.of(context).size.height,
            color: CleanAndroidDesignDefaultColors.background,
          ),
          AnimatedContainer(
            duration: Duration(milliseconds: 150),
            width: showRightSide ? (MediaQuery.of(context).size.width / 2) : 0,
            child: Navigator(
              key: SplitNavigator.rightNavigatorKey,
              onGenerateRoute: (settings) =>
                  NoPageRoute(routeObserver, setShowRightSide),
              observers: [routeObserver],
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildDraggable() {
    if (!showDraggable) return Container();

    if (draggablePosition == null) {
      draggableWidth = MediaQuery.of(context).size.width / 2;
      draggableHeight = MediaQuery.of(context).size.height * 0.8;
      draggablePosition = Offset(MediaQuery.of(context).size.width * 0.2,
          MediaQuery.of(context).size.height * 0.1);
    }

    var draggableChild = AnimatedContainer(
      duration: Duration(milliseconds: 150),
      width: draggableWidth,
      height: draggableHeight,
      child: Navigator(
        key: SplitNavigator.draggableNavigatorKey,
        onGenerateRoute: (settings) =>
            NoPageRoute(draggableRouteObserver, setShowDraggable),
        observers: [draggableRouteObserver],
      ),
    );

    return Positioned(
      left: draggablePosition.dx - draggableWidth / 2,
      top: draggablePosition.dy,
      child: Draggable(
        child: draggableChild,
        childWhenDragging: Container(),
        feedback: draggableChild,
        onDraggableCanceled: (Velocity velocity, Offset offset) {
          setState(() => draggablePosition = offset);
        },
      ),
    );
  }

  void setShowRightSide(bool showRightSide) {
    Future.delayed(
        Duration.zero,
        () => setState(() {
              this.showRightSide = showRightSide;
              Future.delayed(Duration.zero, () {
                SplitNavigator.rightIsShown = showRightSide;
                SplitNavigator.updateKey.update();
              });
            }));
  }

  void setShowDraggable(bool showDraggable) {
    Future.delayed(
        Duration.zero,
        () => setState(() {
              this.showDraggable = showDraggable;
              Future.delayed(Duration.zero, () {
                SplitNavigator.draggableIsShown = showDraggable;
                SplitNavigator.updateKey.update();
              });
            }));
  }
}

class NavigatorListener with RouteAware {
  int n = -1;

  final void Function(bool showNavigator) onChange;

  NavigatorListener(
    this.onChange,
  );

  @override
  void didPush() {
    super.didPush();
    n++;
    if (n == 1) onChange(true);
  }

  @override
  void didPushNext() {
    super.didPushNext();
    n++;
    if (n == 1) onChange(true);
  }

  @override
  void didPop() {
    super.didPop();
    n--;
    if (n == 0) onChange(false);
  }

  @override
  void didPopNext() {
    super.didPopNext();
    n--;
    if (n == 0) onChange(false);
  }
}

class NoPageRoute extends MaterialPageRoute {
  NoPageRoute(
    RouteObserver<PageRoute> routeObserver,
    void Function(bool showNavigator) onChange,
  ) : super(
          builder: (context) {
            routeObserver.subscribe(
                NavigatorListener(onChange), ModalRoute.of(context));
            return Center(
              child: LoadingWidget(),
            );
          },
        );
}
